/*
 * FreeRTOS Kernel V10.2.1
 * Copyright (C) 2019 Amazon.com, Inc. or its affiliates.  All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * http://www.FreeRTOS.org
 * http://aws.amazon.com/freertos
 *
 * 1 tab == 4 spaces!
 */

#include "ISR_Support.h"
#include "iodefine.h"

static void vPortYield(void);
void vPortStartFirstTask(void);
static void vPortTickISR(void);

extern void vTaskSwitchContext(void);
extern uint16_t xTaskIncrementTick(void);

/* FreeRTOS yield handler.  This is installed as the watchdog timer interrupt
 * handler but used as something like a software triggered interrput.
 */
#pragma interrupt vPortYield(vect=INTWDTI)
static void vPortYield(void)
{
	volatile uint16_t SP[1];		/* Perform SP <-- SP-2 / SP <-- SP+2 operation. */

	SP[0] = usCriticalNesting;
	vPortSAVE_CONTEXT_C();			/* Save the context of the current task. */

	vTaskSwitchContext();			/* Call the scheduler to select the next task. */

	vPortRESTORE_CONTEXT_C(); 		/* Restore the context of the next task to run. */
	usCriticalNesting = SP[0];
}

/* Starts the scheduler by restoring the context of the task that will execute
 * first.
 */
#pragma interrupt vPortStartFirstTask
void vPortStartFirstTask(void)
{
	volatile uint16_t SP[1];		/* Perform SP <-- SP-2 / SP <-- SP+2 operation. */

									/* The current context is none of tasks so that */
									/* it is not saved. */

	vPortRESTORE_CONTEXT_C(); 		/* Restore the context of whichever task the ... */
	usCriticalNesting = SP[0];
}									/* An interrupt stack frame is used so the task */
									/* is started using a RETI instruction. */

/* FreeRTOS tick handler.  This is installed as the interval timer interrupt
 * handler.
 */
#pragma interrupt vPortTickISR(vect=INTIT)
static void vPortTickISR(void)
{
	volatile uint16_t SP[1];		/* Perform SP <-- SP-2 / SP <-- SP+2 operation. */

	SP[0] = usCriticalNesting;
	vPortSAVE_CONTEXT_C();			/* Save the context of the current task. */

	if (xTaskIncrementTick() != 0)	/* Call the timer tick function. */
	{
		vTaskSwitchContext();		/* Call the scheduler to select the next task. */
	}

	vPortRESTORE_CONTEXT_C(); 		/* Restore the context of the next task to run. */
	usCriticalNesting = SP[0];
}
